function Feats = m_cmpSegFeats_2D(metaD, binW, posOrNeg)
% Find the rectangle region that yield highest score and extract
% histogram of the region. 
% Inputs:
%   metaD is structure with require fields:
%       featFiles_pos, featFiles_neg: pointers to feature files
%       szRng_pos, szRng_neg: range for postive and negative windows
%       nBranch, nLevel: options for feature computation
%       shldDisp_cmpSegFeat: should display or not?
%       b: value for the offset, to display 'positive' or 'negative' on the figures.
%   binW: the weight vector of SVM
%   posOrNeg: valid options are 'pos' or 'neg'. 
% Outputs:
%   Feats: nBin*n matrix, where n is the number of positive or negative signals
%       depending on posOrNeg option.
% By Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 3 Oct 2010

if strcmpi(posOrNeg, 'pos')
    clusterFiles = metaD.featFiles_pos;
    szRange = metaD.szRng_pos;
    if isfield(metaD, 'imFiles_pos')
        imFiles = metaD.imFiles_pos;
    end;
elseif strcmp(posOrNeg, 'neg')
    clusterFiles = metaD.featFiles_neg;
    szRange = metaD.szRng_neg;
    if isfield(metaD, 'imFiles_neg')
        imFiles = metaD.imFiles_neg;
    end;
else
    error('m_cmpSegFeats_2D: unknown posOrNeg option');
end;

nBranch = metaD.nBranch;
nLevel = metaD.nLevel;
shldDisp = metaD.shldDisp_cmpSegFeat;
b = metaD.b;

    
isKpAssign = 1; % sparse keypoints.
if ~exist('shldDisp', 'var')
    shldDisp = 0;
end;

nIm = length(clusterFiles);

nBin = length(binW);
Feats = zeros(nBin*(nBranch^nLevel - 1)/(nBranch - 1), nIm);

if shldDisp
    nR = ceil(sqrt(nIm));
    nC = nR;
    boxes = zeros(4, nIm); % store the rectangles.
    %         clusterImSzs = zeros(nIm, 2); % store size of cluster images.
    predictLabels = zeros(nIm, 1);
    %         figure;
end;

for i=1:nIm
    clusterIm = m_readFeatFile_2D(clusterFiles{i});
    if exist('option', 'var') && strcmpi(option, 'scramble_random')
        ranNums = randperm(size(clusterIm, 1));
        clusterIm(:,[1,2]) = clusterIm(ranNums, [1,2]); % permute the location of kp
    end;
    
    imH = ceil(max(clusterIm(:,1)));
    imW = ceil(max(clusterIm(:,2)));
    
    
    box = ml_findBox1b(clusterIm, imH, imW, -binW, szRange);
    
    leafHists = ml_cmpLeafHists(clusterIm, nBin, box, nBranch, nLevel, isKpAssign);
    histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
    Feats(:,i) = histPyr(:);
    
    %         % compress (group zero-weight bins) the weight and cluster image
    %         % for faster branch and bound
    %         [clusterIm2, binW2] = m_mexCompressHist(clusterIm, binW, 1e-7);
    %         box = ml_findBox1b(clusterIm2, binW2, szRange);
    %         box = ml_findBox1b(clusterIm2, binW2, 4, 0, szRange, 0);
    %         mask = zeros(size(clusterIm));
    %         mask(box(2):box(4), box(1):box(3)) = 1;
    %         Feats(:,i) = ml_cmpRegHist_multi(clusterIm, nBin, mask);
    
    if shldDisp
        %subplot(nR, nC, i); imagesc(clusterIm); axis image off;
        %drawRect(box, 'r', 2);
        boxes(:,i) = box;
        %             clusterImSzs(i, :) = size(clusterIm);
        if -binW(:)'*Feats(:,i) + b >= 0
            %title('positive');
            predictLabels(i) = 1;
        end;
    end;
end

if shldDisp
    [dc, machineName] = system('uname -n');
    figure('Name', machineName);
    for i=1:nIm
        im = imread(imFiles{i});
        %             im = imresize(im, clusterImSzs(i,:));
        subplot(nR, nC, i); imshow(im); axis image off;
        box = boxes(:,i);
        drawRect(box, 'r', 2);
        if predictLabels(i) == 1
            title('positive');
        end;
    end;
end;

function drawRect(box, color, lineWidth)
    rectangle('Position', [box(1), box(2), box(3) - box(1), box(4) - box(2)], ...
        'LineWidth', lineWidth, 'EdgeColor', color);
