function m_genCMUKitchenData()

checkClustData();
return;

% txt2matData(recipe)

dataDir = '/Users/hoai/DataSets/CMUKitchen/FiveWiredIMUs/';

trainSubIDs = 7:2:55;
testSubIDs = 8:2:54;

% winSzs = [125 250, 500, 750, 1000];
winSzs = 500;
nBins = [50, 100, 200, 250, 500, 1000];

for i=1:length(winSzs)
    fprintf('processing i: %d/%d\n', i, length(winSzs));
    winSz = winSzs(i);
    D = getBaoIntilleFeats(trainSubIDs, testSubIDs, winSz);    
%     save(sprintf('%s/BaoIntille_winSz%d', dataDir, winSz), 'D', 'trainSubIDs', 'testSubIDs', 'winSz');
    load(sprintf('%s/BaoIntille_winSz%d', dataDir, winSz), 'D', 'trainSubIDs', 'testSubIDs', 'winSz');
    for j=1:length(nBins);
        fprintf('processing i: %d/%d, j: %d/%d\n', i, length(winSzs), j, length(nBins));
        nBin = nBins(j);
        [DClust, Centroids] = getClustFeats(D, nBin);
        save(sprintf('%s/BaoIntille_winSz%d_nBin%d', dataDir, winSz, nBin), ...
            'DClust', 'Centroids', 'nBin');
    end;
end;


% Check if the clustering model works properly.
function checkClustData()
    winSz = 1000;
    nBin = 50;

    dataDir = '/Users/hoai/DataSets/CMUKitchen/FiveWiredIMUs/';
    load(sprintf('%s/BaoIntille_winSz%d_nBin%d.mat', dataDir, winSz, nBin), 'DClust');    
    A = zeros(nBin, 100);
    cnt = 0;
    for i=1:5
        for j=1:length(DClust(i).tstTSs)
            cnt = cnt + 1;
            A(:,cnt) = hist(DClust(i).tstTSs{j}(:,2), 1:nBin);
        end;
    end;
    A = A(:,1:cnt);
    A = A./repmat(sum(A, 2), 1, cnt);
    figure(3); subplot(1,2,2); imagesc(A);
    
    
    keyboard;

    


% function tuneKitchenData()



function [DClust, Centroids] = getClustFeats(D, nBin)
    
    nRecipe = length(D);
    TrD = cell(1, nRecipe);
    TstD = cell(1, nRecipe);
    for i=1:nRecipe
        TrD{i}  = cat(2, D(i).trTSs{:});
        TstD{i} = cat(2, D(i).tstTSs{:});
    end;
    TrD     = cat(2, TrD{:});
    TstD    = cat(2, TstD{:});
    
    mxVals = max(max(abs(TrD), [], 2), max(abs(TstD), [], 2));
    TrD     = TrD./repmat(mxVals, 1, size(TrD, 2)); % normalize the attributes
    TstD    = TstD./repmat(mxVals, 1, size(TstD, 2));

    Centroids = yael_kmeans(single(cat(2, TrD, TstD)), nBin);
    Centroids = double(Centroids);
    
    for i=1:nRecipe
        DClust(i).recipe = D(i).recipe;
        n_i = length(D(i).trTSs); % number of time series for this recipe
        DClust(i).trTSs = cell(1, n_i);
        for j=1:n_i
            len_j = size(D(i).trTSs{j}, 2);
            Dist = ml_sqrDist(Centroids, D(i).trTSs{j}./repmat(mxVals, 1, len_j));
            [dc, I] = min(Dist, [], 1);
            DClust(i).trTSs{j} = [1:len_j; I]';
        end;
        
        n_i = length(D(i).tstTSs); % number of time series for this recipe
        DClust(i).tstTSs = cell(1, n_i);
        for j=1:n_i
            len_j = size(D(i).tstTSs{j},2);
            Dist = ml_sqrDist(Centroids, D(i).tstTSs{j}./repmat(mxVals, 1, len_j));
            [dc, I] = min(Dist, [], 1);
            DClust(i).tstTSs{j} = [1:len_j; I]';
        end;
    end;
    

% Get Bao-Intille features
function D = getBaoIntilleFeats(trainSubIDs, testSubIDs, winSz)
    % split into train and test sets, old for training and even for testing.
    recipes = {'Brownie', 'Eggs', 'Pizza', 'Salad', 'Sandwich'};

    dataDir = '/Users/hoai/DataSets/CMUKitchen/FiveWiredIMUs/';

    for i=1:length(recipes)
        recipe = recipes{i};
        dataPath = [dataDir recipe];
        fprintf('   processing recipet %s\n', recipe);
        D(i).trTSs = genBaoIntilleFeatFiles_helper(trainSubIDs, winSz, dataPath);
        D(i).tstTSs = genBaoIntilleFeatFiles_helper(testSubIDs, winSz, dataPath);
        D(i).recipe = recipe;
    end;


function Feats = genBaoIntilleFeatFiles_helper(subIDs, winSz, dataPath)
    stepSz = 125;
    cnt = 0;
    for i=1:length(subIDs)
        subID = subIDs(i);
        fprintf('      processing subID: %d\n', subID);
        fileName = sprintf('%s/S%02d.mat', dataPath, subID);
        if exist(fileName, 'file')
            cnt = cnt + 1;
            load(fileName, 'D');
            Feats{cnt} = m_cmpBaoIntilleFeats(D, winSz, stepSz);
        end;
    end;




% convert from text files to mat files.
function txt2matData(recipe)
    dataPath = ['/Users/hoai/DataSets/CMUKitchen/FiveWiredIMUs/' recipe];
    subIDs = 7:55;
    sensorIDs = [2794 3337 2796 3261 2795]; % left-arm, right-arm, left-leg, right-leg, back
    sensorOrders = {'left-arm', 'right-arm', 'left-leg', 'right-leg', 'back'};

    nSensor = length(sensorIDs);
    for i=1:length(subIDs)
        subID = subIDs(i);
        fprintf('processing subject %d\n', subID);
        dataDir = [dataPath, sprintf('/S%02d', subID)]; 
        if exist(dataDir, 'dir')
            minNumFrm = inf;
            Ds = cell(1, nSensor);
            for j=1:nSensor
                senID = sensorIDs(j);
                dataFile = [dataDir, '/out_', num2str(senID)];
                Ds{j} = load(dataFile);
                Ds{j} = Ds{j}';
                minNumFrm = min(minNumFrm, size(Ds{j}, 2));
            end;
            for j=1:nSensor
                Ds{j} = Ds{j}(:, 1:minNumFrm);
            end;
            D = cat(1, Ds{:});
            save([dataPath, '/', sprintf('/S%02d.mat', subID)], 'D', 'sensorOrders', 'recipe');
        end;  
    end;



