function m_main_synData1D_MC(nSeg)
% Run experiment on 1D synthetic data. 
% This is for the multi-class case.
% Inputs:
%   nSeg: maximum number of sub-segments to be considered.
% By Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 7 July 2011

addpath('/Users/hoai/Study/Projects/SegMMC/bin/');
if ~exist('nSeg', 'var')
    nSeg = 3;
end

% Generate very simple synthetic data
nTr = 30; % number of train
nTst = 100;
nClass = 5;
nBin = 10;
[TrD, trLb] = genRandomData(nClass, nBin, nTr);
[TstD, tstLb] = genRandomData(nClass, nBin, nTst);
% [TrD, trLb] = genDebugData(nClass, nBin, nTr);
% [TstD, tstLb] = genDebugData(nClass, nBin, nTst);


trMetaD.C = 10;
trMetaD.nBin = nBin;
trMetaD.nClass = nClass;
trMetaD.nSeg = nSeg;
trMetaD.cmpInitFeats = 'm_cmpInitFeats_1D_MC';
trMetaD.cmpSegFeats  = 'm_cmpSegFeats_1D_MC';
trMetaD.runID = '1DsynData_MC';

% Do classification using global statistics
trMetaD.initOption_pos = 'Global';
trMetaD.initOption_neg = 'Global';
trMetaD.D = TrD;
trMetaD.lb = trLb;
trMetaD.nSample = size(trMetaD.D,2);
TrGlobFeats  = m_cmpInitFeats_1D_MC(trMetaD, 'pos'); 
Ws = m_mexMCSVM(TrGlobFeats, trMetaD.nClass, trMetaD.C, trLb);


tstMetaD   = trMetaD;
tstMetaD.D = TstD;
tstMetaD.lb = [];
tstMetaD.nSample = size(TstD, 2); 

TrFeats_glob = m_cmpInitFeats_1D_MC(trMetaD, 'pos'); 
TrSVMScores_glob = Ws'*TrFeats_glob;
[dc, predLb_tr_glob] = max(TrSVMScores_glob, [], 1);
[ConfMat_tr_glob, acc_tr_glob] = m_getConfMat(nClass, trLb, predLb_tr_glob); 

TstFeats_glob = m_cmpInitFeats_1D_MC(tstMetaD, 'pos'); 
TstSVMScores_glob = Ws'*TstFeats_glob;
[dc, predLb_tst_glob] = max(TstSVMScores_glob, [], 1);
[ConfMat_tst_glob, acc_tst_glob] = m_getConfMat(nClass, tstLb, predLb_tst_glob); 


trMetaD.initOption_pos = 'random';
trMetaD.initOption_neg = 'random';
trMetaD.initW = Ws;
[Ws_seg, objVals, wDeviations] = m_learnSegMCSVM(trMetaD);

[TrFeats_seg, predLb_tr_seg] = m_cmpSegFeats_1D_MC(trMetaD, Ws_seg, 'test');
[ConfMat_tr_seg, acc_tr_seg] = m_getConfMat(nClass, trLb, predLb_tr_seg);

[TstFeats_seg, predLb_tst_seg] = m_cmpSegFeats_1D_MC(tstMetaD, Ws_seg, 'test');
[ConfMat_tst_seg, acc_tst_seg] = m_getConfMat(nClass, tstLb, predLb_tst_seg);


fprintf('Global_train, mean acc: %g, confusion matrix:\n', acc_tr_glob); disp(ConfMat_tr_glob);
fprintf('SegSVM_train, mean acc: %g, confusion matrix:\n', acc_tr_seg); disp(ConfMat_tr_seg);


fprintf('Global_tst, mean acc: %g, confusion matrix:\n', acc_tst_glob); disp(ConfMat_tst_glob);
fprintf('SegSVM_tst, mean acc: %g, confusion matrix:\n', acc_tst_seg); disp(ConfMat_tst_seg);


function [D, lb] = genRandomData(nClass, nBin, n)
    tsLen = 200; % time series length
    nFgSeg = 3; % number of foreground segment
    fgSegLen = 10; % length of foreground segment

    lb = randi(nClass, [1, n]);
    lb = sort(lb);
    D = cell(1, n);
    for i=1:n
%         signal = randi([nClass + 1, nBin], [1, tsLen]);
        signal = randi(nBin, [1, tsLen]);
        evStarts = randi(tsLen - fgSegLen + 1, [1, nFgSeg]);
        for j=1:nFgSeg
            signal(evStarts(j):(evStarts(j) + fgSegLen - 1)) = lb(i);
        end;
        D{i} = [1:tsLen; signal]';
    end;
%     figure(1); nR = 5; nC = 2;
%     for i=1:(nR*nC)
%         subplot(nR, nC, i); plot(D{i}(2,:));
%     end;

function [D, lb] = genDebugData(nClass, nBin, n)
    tsLen = 70; % time series length
    nFgSeg = 3; % number of foreground segment
    fgSegLen = 5; % length of foreground segment

    m = floor(n/nClass);
    lb = ones(1, n);
    for i=1:nClass
        lb(1+(i-1)*m:i*m) = i;
    end;
        
    D = cell(1, n);
    for i=1:n
        signal = (nClass+1)*ones(1, tsLen);
%         evStarts = fgSegLen + 2*fgSegLen*(0:(nFgSeg-1));
        evStarts = randi(tsLen - fgSegLen + 1, [1, nFgSeg]);

        for j=1:nFgSeg
            signal(evStarts(j):(evStarts(j) + fgSegLen - 1)) = lb(i);
        end;
        D{i} = [1:tsLen; signal]';
    end;
