/*
 * m_mexLearnSegMCSVM.cpp
 *
 * Created on: 1 Oct 2010
 * Last modified: 1 Oct 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_utils.h"
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gsl/gsl_cblas.h>
#include <ilcplex/ilocplex.h>
#include "m_SegMCSVM_CCCP.cpp"

using namespace std;

class SegMCSVMFeatGen{

public:
   SegMCSVMFeatGen(const mxArray *metaD_, int nBin_, int nClass_, int nSample_){
      metaD    = (mxArray *) metaD_;
      nBin     = nBin_;
      nClass   = nClass_;
      nSample  = nSample_;
      wSz      = nBin*nClass;
      int index = 0;
      mxGetString(mxGetField(metaD, index, "cmpInitFeats"), cmpInitFeats_funcName, 100);
      mxGetString(mxGetField(metaD, index, "cmpSegFeats"),  cmpSegFeats_funcName, 100);
      mexPrintf("cmmInitFeats: %s, cmpSegFeats: %s\n", cmpInitFeats_funcName, cmpSegFeats_funcName);
   };

   void getSegFeats(IloNumArray w, const char *posOrNeg, double *feats, int *classIDs = NULL){
      mxArray *results[2];
      mxArray *args[3];
      args[0] = metaD;
      args[1] = mxCreateDoubleMatrix(nBin, nClass, mxREAL);
      double *argPtr = mxGetPr(args[1]); //assign w to the args[1]
      for(int i = 0;i < wSz; i++) argPtr[i] = w[i];
      args[2] = mxCreateString(posOrNeg);
      mexCallMATLAB(2, results, 3, args, cmpSegFeats_funcName);
      memcpy(feats, mxGetPr(results[0]), nBin*nSample*sizeof(double));
      if (!strcmp(posOrNeg, "neg")){
         double *classIDs_ptr = mxGetPr(results[1]);
         for (int i=0; i < nSample; i++) classIDs[i] = (int) *classIDs_ptr++ -1;
      }
   }


   void getInitFeats(const char *posOrNeg, double *feats, int *classIDs = NULL){
      mxArray *results[2];
      mxArray *args[2];
      args[0] = metaD;
      args[1] = mxCreateString(posOrNeg);
      mexCallMATLAB(2, results, 2, args, cmpInitFeats_funcName);

      memcpy(feats, mxGetPr(results[0]), nBin*nSample*sizeof(double));
      if (!strcmp(posOrNeg, "neg")){
         double *classIDs_ptr = mxGetPr(results[1]);
         for (int i=0; i < nSample; i++) classIDs[i] = (int) *classIDs_ptr++ -1;
      }
   }

private:
   mxArray *metaD;
   int nBin, nClass, wSz, nSample;
   char cmpInitFeats_funcName[100];
   char cmpSegFeats_funcName[100];
};

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 1) mexErrMsgTxt("1 input argument is required");
   if (nlhs>5) mexErrMsgTxt("Too many outputs");

   int METAD_IN = 0;
   const mxArray *metaD = prhs[METAD_IN];

   int index = 0;
   double C = mxGetScalar(mxGetField(metaD, index, "C"));
   int nBin = (int) mxGetScalar(mxGetField(metaD, index, "nBin"));
   int nClass = (int) mxGetScalar(mxGetField(metaD, index, "nClass"));
   mxArray *lb_mxarray = mxGetField(metaD, index, "lb");
   int nSample = mxGetNumberOfElements(lb_mxarray);
   double *lb_ptr = (double *) mxGetPr(lb_mxarray);
   int *lb = (int *) mxCalloc(nSample, sizeof(int));
   for (int i=0; i < nSample; i++) lb[i] = (int) *lb_ptr++ - 1;

   char runID[100];
   mxGetString(mxGetField(metaD, index, "runID"), runID, 100);
   mexPrintf("runID: %s, C: %g, nBin: %d, nClass: %d, nSample: %d\n", runID, C, nBin, nClass, nSample);

   /* Optimization */
   SegMCSVMFeatGen featGen(metaD, nBin, nClass, nSample);
   SegMCSVM_OptRslt optRslt = SegMCSVM_CCCP<SegMCSVMFeatGen>(featGen, lb, C, nBin, nClass, nSample, runID);

   /* Create the out going Arrays  */
   int W_OUT = 0;
   int OBJVALS_OUT = 1;
   int WDEVIATION_OUT = 2;
   int ALL_WS_OUT = 3;

   plhs[W_OUT] = mxCreateDoubleMatrix(nBin, nClass, mxREAL);
   double *w_outPtr = mxGetPr(plhs[W_OUT]);
   for (int i=0; i < nBin*nClass; i++) w_outPtr[i] = optRslt.w[i];

   int nIter1 = optRslt.nIter1;
   plhs[OBJVALS_OUT]    = mxCreateCellMatrix(1, nIter1);
   plhs[WDEVIATION_OUT] = mxCreateCellMatrix(1, nIter1);
   plhs[ALL_WS_OUT] = mxCreateCellMatrix(1, nIter1);
   for (int i=0; i < nIter1; i++){
      int nIter2 = optRslt.nCGIters[i];
      mxArray *objVals_out = mxCreateDoubleMatrix(nIter2,1 , mxREAL);
      double *objVals_outPtr = mxGetPr(objVals_out);
      mxArray *wDeviation_out = mxCreateDoubleMatrix(nIter2,1 , mxREAL);
      double *wDeviation_outPtr = mxGetPr(wDeviation_out);

      for (int j=0; j < nIter2; j++) {
         objVals_outPtr[j] = optRslt.objVals[i][j];
         wDeviation_outPtr[j] = optRslt.wDeviation[i][j];
      }
      mxSetCell(plhs[OBJVALS_OUT], i, objVals_out);
      mxSetCell(plhs[WDEVIATION_OUT], i, wDeviation_out);

      mxArray *all_ws_out = mxCreateDoubleMatrix(nBin, nClass, mxREAL);
      double *all_ws_out_ptr = mxGetPr(all_ws_out);
      memcpy(all_ws_out_ptr, optRslt.all_ws + i*nBin*nClass, nBin*nClass*sizeof(double));
      mxSetCell(plhs[ALL_WS_OUT], i, all_ws_out);
   }
}
