/*
 * m_mexLearnSegSVM_1D.cpp
 *
 * Created on: 1 Oct 2010
 * Last modified: 1 Oct 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_utils.h"
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gsl/gsl_cblas.h>
#include <ilcplex/ilocplex.h>
#include "m_SegSVM_CCCP.cpp"

using namespace std;

class SegSVMFeatGen{

public:
   SegSVMFeatGen(const mxArray *metaD_, int nBin_){
      metaD = (mxArray *) metaD_;
      nBin = nBin_;
      int index = 0;
      mxGetString(mxGetField(metaD, index, "cmpInitFeats"), cmpInitFeats_funcName, 100);
      mxGetString(mxGetField(metaD, index, "cmpSegFeats"),  cmpSegFeats_funcName, 100);
      mexPrintf("cmmInitFeats: %s, cmpSegFeats: %s\n", cmpInitFeats_funcName, cmpSegFeats_funcName);
   };

   double *getInitNegFeats(){
      return getInitFeats("neg");
   }

   double *getInitPosTrFeats(){
      return getInitFeats("pos");
   }


   double *getNegFeats(IloNumArray w){
      return getSegFeats(w, "neg");
   }

   double *getPosFeats(IloNumArray w){
      return getSegFeats(w, "pos");
   }


private:
   mxArray *metaD;
   int nBin;
   char cmpInitFeats_funcName[100];
   char cmpSegFeats_funcName[100];

   double *getSegFeats(IloNumArray w, char const *posOrNeg){
      mxArray *result;
      mxArray *args[3];
      args[0] = metaD;
      args[1] = mxCreateDoubleMatrix(nBin, 1, mxREAL);
      double *argPtr = mxGetPr(args[1]); //assign -w to the args[1]
      for(int i = 0;i < nBin;i++) argPtr[i] = w[i];

      args[2] = mxCreateString(posOrNeg);
      mexCallMATLAB(1, &result, 3, args, cmpSegFeats_funcName);
      return mxGetPr(result);
   }

   double *getInitFeats(char const *posOrNeg){
      mxArray *result;
      mxArray *args[2];
      args[0] = metaD;
      args[1] = mxCreateString(posOrNeg);
      mexCallMATLAB(1,&result,2,args, cmpInitFeats_funcName); //Generate initial negative constraints
      return mxGetPr(result);
   }
};

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 1)
      mexErrMsgTxt("1 input argument is required");
   if (nlhs>5)
      mexErrMsgTxt("Too many outputs");

   int METAD_IN = 0;
   const mxArray *metaD = prhs[METAD_IN];

   int index = 0;
   double C = mxGetScalar(mxGetField(metaD, index, "C"));
   int nNeg = (int) mxGetScalar(mxGetField(metaD, index, "nNeg"));
   int nPos = (int) mxGetScalar(mxGetField(metaD, index, "nPos"));
   int nBin = (int) mxGetScalar(mxGetField(metaD, index, "nBin"));
   bool useBias = (bool) mxGetScalar(mxGetField(metaD, index, "useBias"));


   char runID[100];
   mxGetString(mxGetField(metaD, index, "runID"), runID, 100);

   mexPrintf("C: %g, nNeg: %d, nPos: %d, nBin: %d, useBias: %d, runID: %s#\n", C, nNeg, nPos, nBin, useBias, runID);

   /* Optimization */
   SegSVMFeatGen featGen(metaD, nBin);
   SegSVM_OptRslt optRslt = SegSVM_CCCP<SegSVMFeatGen>(featGen, C, nPos, nNeg, nBin, useBias, runID);

   /* Create the out going Arrays  */
   int W_OUT = 0;
   int B_OUT = 1;
   int OBJVALS_OUT = 2;
   int WDEVIATION_OUT = 3;

   plhs[W_OUT] = mxCreateDoubleMatrix(nBin, 1, mxREAL);
   double *w_outPtr = mxGetPr(plhs[W_OUT]);
   for (int i=0; i < nBin; i++) w_outPtr[i] = optRslt.w[i];

   plhs[B_OUT] = mxCreateDoubleScalar(optRslt.b);

   int nIter1 = optRslt.nIter1;
   plhs[OBJVALS_OUT]    = mxCreateCellMatrix(1, nIter1);
   plhs[WDEVIATION_OUT] = mxCreateCellMatrix(1, nIter1);
   for (int i=0; i < nIter1; i++){
      int nIter2 = optRslt.nCGIters[i];
      mxArray *objVals_out = mxCreateDoubleMatrix(nIter2,1 , mxREAL);
      double *objVals_outPtr = mxGetPr(objVals_out);
      mxArray *wDeviation_out = mxCreateDoubleMatrix(nIter2,1 , mxREAL);
      double *wDeviation_outPtr = mxGetPr(wDeviation_out);

      for (int j=0; j < nIter2; j++) {
         objVals_outPtr[j] = optRslt.objVals[i][j];
         wDeviation_outPtr[j] = optRslt.wDeviation[i][j];
      }
      mxSetCell(plhs[OBJVALS_OUT], i, objVals_out);
      mxSetCell(plhs[WDEVIATION_OUT], i, wDeviation_out);
   }
}
